#include "vcpu.h"
#include "mem.h"
#include "screen.h"
#include <stdio.h>

// Current VCPU state
VCPU_State State;

void VCPUInit(){
  State.PC=ADDR_ROM;
  State.S=0;
}

void VCPUFetch(){
  unsigned char byte[2];
  MemRead(byte,2,State.PC); // Little indian to -1 no +1
  State.opcode=byte[0];
  State.opcode=State.opcode<<8;
  State.opcode=State.opcode | byte[1];
  State.PC+=2;
}

void VCPUDecode(){
  State.X=(State.opcode>>8) & 0xF;
  State.Y=(State.opcode>>4) & 0xF;
  State.N=State.opcode & 0xF;

  State.NN=State.Y;
  State.NN=State.NN<<4;
  State.NN=State.NN | State.N;

  State.NNN=State.opcode&0x0FFF;
}

void VCPUExecute(){
  //  VCPUDump();
  switch(State.opcode >> 12){
  case 0x0:
    ScreenClear();
    break
      ;;
  case 0x1:
    State.PC=State.NNN;
    break
      ;;
  case 0x6:
    State.V[State.X]=State.NN;
    break
      ;;
  case 0x7:
    State.V[State.X]+=State.NN;
    break
      ;;
  case 0xA:
    State.I=State.NNN;
    break
      ;;
  case 0xD: // Draw a sprite
    int X=State.V[State.X]%63;
    int Y=State.V[State.Y]%31;
    State.V[REG_FLAG]=0; // Set flag to 0
    int width,height;
    ScreenWH(&width,&height);
    for(char row=0;row<State.N;row++){
      // Stop if row out of screen
      if(Y+row>=height)
        break;
      char sprite;
      MemRead(&sprite,1,State.I+row); // Load sprite
      // Draw sprite
      for(int shift=0;shift<8;shift++){
        // Stop if column is out of screen
        if(X+shift >= width)
          break;
        if(ScreenPixelApply(X+shift,Y+row,(sprite>>(7-shift))&0x1))
          State.V[REG_FLAG]=1;
      }
    }
    break;
    ;;
  }
}

void VCPUDump(){
  printf("opcode: 0x%04x\n",State.opcode&0xFFFF);
  printf("X: 0x%01x\n",State.X);
  printf("Y: 0x%01x\n",State.Y);
  printf("N: 0x%01x\n",State.N);
  printf("NN: 0x%02x\n",State.NN);
  printf("NNN: 0x%03x\n",State.NNN);
}
